#include <stdio.h>

/* 
-------------------------------------------------------------------------------
Propagacio a terra de la contaminacio radioactiva 
-------------------------------------------------------------------------------
Dades: dt (mida del pas en dies), imax (passos que es volen calcular), jmax
       (nombre d'iteracions per estabilitzar), ji (concentracio a l'aire),
       pluja (volum de pluja que cau en un dia), ym (densitat de vegetacio),
       crei (tant per u de creixement diari).
Resultats: Al fitxer "cesi1.dat". Cada fila representa un instant de temps 
           La primera columma es el temps en dies. La segona, tercera i
           quarta son la concentracio en Bq/m^2 a la capa mes superficial,
           la intermitja i al subsol, respectivament. La cinquena i sisena
           son la concentracio en Bq/kg a l'exterior i a l'interior de les
           plantes respectivament.
Dades de prova: dt=0.05 imax=10 jmax=2 ji=0.0001 pluja=1 ym=1 crei=0 ==>
                0.05 0.00299313 4.97353e-08 0 0.000995609 4.60484e-06
                0.1 0.00598933 1.99042e-07 8.60639e-13 0.00198805 9.21443e-06
                0.15 0.00898857 4.48021e-07 3.44364e-12 0.00297732 1.38287e-05
                0.2 0.0119909 7.96771e-07 8.61159e-12 0.00396345 1.84478e-05
                0.25 0.0149962 1.24539e-06 1.72279e-11 0.00494644 2.30715e-05
                0.3 0.0180045 1.79399e-06 3.0157e-11 0.00592629 2.76999e-05
                0.35 0.0210158 2.44265e-06 4.82639e-11 0.00690303 3.2333e-05
                0.4 0.0240302 3.19148e-06 7.24148e-11 0.00787665 3.69707e-05
                0.45 0.0270475 4.04058e-06 1.03476e-10 0.00884717 4.16131e-05
                0.5 0.0300678 4.99004e-06 1.42316e-10 0.00981461 4.626e-05
Observacions: No n'hi ha prou amb que les unitats d'entrada siguin coherents.
              Les unitats venen fixades per les unitats de les variables que
              son a dins del programa.
              Faran falta molt mes de 10 passos per arribar a veure la
              estabilitazacio dels valors.
              Tan sols el valor jmax=1 es justificable des del punt de vista
              teoric.
-------------------------------------------------------------------------------
*/

main()
{

int i,j,k,l,imax,jmax;
float dx[4],x0[4],x[4],xf[4],xm[4],font[4],mat[4][4],dt,t0,t,tf,tm,cont,x4,
        depo,ym,crei,ji,pluja;
FILE *arxiu;

printf("Mida del pas en dies: ");
scanf("%f",&dt);
printf("Passos que vols calcular: ");
scanf("%d",&imax);
printf("Iteracions per estabilitzar: ");
scanf("%d",&jmax);
printf("Concentracio a l'aire en Bq/m3: ");
scanf("%f",&ji);
printf("Quantitat de pluja en mm/dia: ");
scanf("%f",&pluja);
printf("Densitat de la vegetacio en Kg/m2: ");
scanf("%f",&ym);
printf("Quin factor creix per dia: ");
scanf("%f",&crei);
depo=ji*(pluja*3600.0*(1.0e3)*(1.64e-4)+(2.4e-3)*3600.0*24.0);
arxiu=fopen("cesi1.dat","w");
/* Suposem que inicialment no hi ha contaminacio */
for(k=0;k<4;k++)x0[k]=0.0;
t0=0;
/* Inicialitzem MAT i FONT ... */
for(k=0;k<4;k++)for(l=0;l<4;l++)mat[k][l]=0.0;
for(k=0;k<4;k++)font[k]=0.0;
/* ... i les omplim amb parametres empirics */
mat[0][0]=-6.33e-5-6.65e-4-8.64e-5;
mat[0][3]=3.0e-2+3.4e-2*pluja;
font[0]=depo*0.75;
mat[1][0]=6.65e-4;
mat[1][1]=-6.33e-5-1.73e-4;
mat[2][1]=1.73e-4;
mat[2][2]=-6.33e-5;
mat[3][0]=8.64e-5;
mat[3][3]=-6.33e-5-3.0e-2-3.4e-2*pluja-crei;
font[3]=depo*0.25;
/* Ara inicialitzem les matrius */
for(i=0;i<4;i++)
	{
	x[i]=x0[i];
	xf[i]=x0[i];
	}
t=t0;
for(i=0;i<imax;i++)   
   {
   tm=t+dt/2.0;
   for(j=0;j<jmax;j++)
      {
      for(k=0;k<4;k++)xm[k]=(x[k]+xf[k])/2.0;
      for(k=0;k<4;k++)
         {		
         cont=0.0;
         for(l=0;l<4;l++)cont=cont+mat[k][l]*xm[l];
         cont=cont+font[k];
         dx[k]=cont*dt;
         xf[k]=x[k]+dx[k];
         }         
      }
   for(k=0;k<4;k++)x[k]=xf[k];
   t=t+dt;         
/* Fem el calcul estatic, tambe amb parametres empirics */
   x4=(2.0e-2)*(x[0]/13.0+x[1]/52.0); 
/* I ho treiem per fitxer */
   fprintf(arxiu,"%g %g %g %g %g %g\n",t,x[0],x[1],x[2],x[3]/ym,x4);
   }
fclose(arxiu);
}
